@@ -326,6 +326,28 @@ timestr(
 	}
 }
 
+/*
+ * Convert from a pair of arbitrary user strings into a timespec.
+ */
+
+int
+timespec_from_string(
+	const char	* secs,
+	const char	* nsecs,
+	struct timespec	* ts)
+{
+	char* p;
+	if (!secs || !nsecs || !ts)
+		return 1;
+	ts->tv_sec = strtoull(secs, &p, 0);
+	if (*p)
+		return 1;
+	ts->tv_nsec = strtoull(nsecs, &p, 0);
+	if (*p)
+		return 1;
+	return 0;
+}
+
 /*
  * Convert from arbitrary user strings into a numeric ID.
  * If it's all numeric, we convert that inplace, else we do
@@ -366,7 +388,7 @@ uid_from_string(
 	char		*sp;
 
 	uid_long = strtoul(user, &sp, 10);
-	if (sp != user) {
+	if (sp != user && *sp == '\0') {
 		if ((uid_long == ULONG_MAX && errno == ERANGE)
 				|| (uid_long > (uid_t)-1))
 			return -1;
@@ -387,7 +409,7 @@ gid_from_string(
 	char		*sp;
 
 	gid_long = strtoul(group, &sp, 10);
-	if (sp != group) {
+	if (sp != group && *sp == '\0') {
 		if ((gid_long == ULONG_MAX && errno == ERANGE)
 				|| (gid_long > (gid_t)-1))
 			return -1;
